/**
 * backbone model definitions for MHM OS
 */

/**
 * Use emulated HTTP if the server doesn't support PUT/DELETE or application/json requests
 */
Backbone.emulateHTTP = false;
Backbone.emulateJSON = false;

var model = {};

/**
 * long polling duration in miliseconds.  (5000 = recommended, 0 = disabled)
 * warning: setting this to a low number will increase server load
 */
model.longPollDuration = 0;

/**
 * whether to refresh the collection immediately after a model is updated
 */
model.reloadCollectionOnModelUpdate = true;


/**
 * a default sort method for sorting collection items.  this will sort the collection
 * based on the orderBy and orderDesc property that was used on the last fetch call
 * to the server. 
 */
model.AbstractCollection = Backbone.Collection.extend({
	totalResults: 0,
	totalPages: 0,
	currentPage: 0,
	pageSize: 0,
	orderBy: '',
	orderDesc: false,
	lastResponseText: null,
	lastRequestParams: null,
	collectionHasChanged: true,
	
	/**
	 * fetch the collection from the server using the same options and 
	 * parameters as the previous fetch
	 */
	refetch: function() {
		this.fetch({ data: this.lastRequestParams })
	},
	
	/* uncomment to debug fetch event triggers
	fetch: function(options) {
            this.constructor.__super__.fetch.apply(this, arguments);
	},
	// */
	
	/**
	 * client-side sorting baesd on the orderBy and orderDesc parameters that
	 * were used to fetch the data from the server.  Backbone ignores the
	 * order of records coming from the server so we have to sort them ourselves
	 */
	comparator: function(a,b) {
		
		var result = 0;
		var options = this.lastRequestParams;
		
		if (options && options.orderBy) {
			
			// lcase the first letter of the property name
			var propName = options.orderBy.charAt(0).toLowerCase() + options.orderBy.slice(1);
			var aVal = a.get(propName);
			var bVal = b.get(propName);
			
			if (isNaN(aVal) || isNaN(bVal)) {
				// treat comparison as case-insensitive strings
				aVal = aVal ? aVal.toLowerCase() : '';
				bVal = bVal ? bVal.toLowerCase() : '';
			} else {
				// treat comparision as a number
				aVal = Number(aVal);
				bVal = Number(bVal);
			}
			
			if (aVal < bVal) {
				result = options.orderDesc ? 1 : -1;
			} else if (aVal > bVal) {
				result = options.orderDesc ? -1 : 1;
			}
		}
		
		return result;

	},
	/**
	 * override parse to track changes and handle pagination
	 * if the server call has returned page data
	 */
	parse: function(response, options) {

		// the response is already decoded into object form, but it's easier to
		// compary the stringified version.  some earlier versions of backbone did
		// not include the raw response so there is some legacy support here
		var responseText = options && options.xhr ? options.xhr.responseText : JSON.stringify(response);
		this.collectionHasChanged = (this.lastResponseText != responseText);
		this.lastRequestParams = options ? options.data : undefined;
		
		// if the collection has changed then we need to force a re-sort because backbone will
		// only resort the data if a property in the model has changed
		if (this.lastResponseText && this.collectionHasChanged) this.sort({ silent:true });
		
		this.lastResponseText = responseText;
		
		var rows;

		if (response.currentPage) {
			rows = response.rows;
			this.totalResults = response.totalResults;
			this.totalPages = response.totalPages;
			this.currentPage = response.currentPage;
			this.pageSize = response.pageSize;
			this.orderBy = response.orderBy;
			this.orderDesc = response.orderDesc;
		} else {
			rows = response;
			this.totalResults = rows.length;
			this.totalPages = 1;
			this.currentPage = 1;
			this.pageSize = this.totalResults;
			this.orderBy = response.orderBy;
			this.orderDesc = response.orderDesc;
		}

		return rows;
	}
});

/**
 * Agenda Backbone Model
 */
model.AgendaModel = Backbone.Model.extend({
	urlRoot: 'api/agenda',
	idAttribute: 'id',
	id: '',
	data: '',
	titulo: '',
	descricao: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'data': new Date(),
		'titulo': '',
		'descricao': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * Agenda Backbone Collection
 */
model.AgendaCollection = model.AbstractCollection.extend({
	url: 'api/agendas',
	model: model.AgendaModel
});

/**
 * Cidades Backbone Model
 */
model.CidadesModel = Backbone.Model.extend({
	urlRoot: 'api/cidades',
	idAttribute: 'id',
	id: '',
	nome: '',
	uf: '',
	defaults: {
		'id': null,
		'nome': '',
		'uf': ''
	}
});

/**
 * Cidades Backbone Collection
 */
model.CidadesCollection = model.AbstractCollection.extend({
	url: 'api/cidadeses',
	model: model.CidadesModel
});

/**
 * EnqPerg Backbone Model
 */
model.EnqPergModel = Backbone.Model.extend({
	urlRoot: 'api/enqperg',
	idAttribute: 'codigo',
	codigo: '',
	pergunta: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'codigo': null,
		'pergunta': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * EnqPerg Backbone Collection
 */
model.EnqPergCollection = model.AbstractCollection.extend({
	url: 'api/enqpergs',
	model: model.EnqPergModel
});

/**
 * EnqResp Backbone Model
 */
model.EnqRespModel = Backbone.Model.extend({
	urlRoot: 'api/enqresp',
	idAttribute: 'codigo',
	codigo: '',
	codPerg: '',
	resposta: '',
	votos: '',
	cor: '',
	tipo: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'codigo': null,
		'codPerg': '',
		'resposta': '',
		'votos': '',
		'cor': '',
		'tipo': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * EnqResp Backbone Collection
 */
model.EnqRespCollection = model.AbstractCollection.extend({
	url: 'api/enqresps',
	model: model.EnqRespModel
});

/**
 * Eventos Backbone Model
 */
model.EventosModel = Backbone.Model.extend({
	urlRoot: 'api/eventos',
	idAttribute: 'id',
	id: '',
	nome: '',
	data: '',
	descricao: '',
	defaults: {
		'id': null,
		'nome': '',
		'data': new Date(),
		'descricao': ''
	}
});

/**
 * Eventos Backbone Collection
 */
model.EventosCollection = model.AbstractCollection.extend({
	url: 'api/eventoses',
	model: model.EventosModel
});

/**
 * Guia Backbone Model
 */
model.GuiaModel = Backbone.Model.extend({
	urlRoot: 'api/guia',
	idAttribute: 'id',
	id: '',
	idTp: '',
	nome: '',
	descricao: '',
	endereco: '',
	fone: '',
	email: '',
	link: '',
	ativo: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'idTp': '',
		'nome': '',
		'descricao': '',
		'endereco': '',
		'fone': '',
		'email': '',
		'link': '',
		'ativo': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * Guia Backbone Collection
 */
model.GuiaCollection = model.AbstractCollection.extend({
	url: 'api/guias',
	model: model.GuiaModel
});

/**
 * Guiatp Backbone Model
 */
model.GuiatpModel = Backbone.Model.extend({
	urlRoot: 'api/guiatp',
	idAttribute: 'id',
	id: '',
	nome: '',
	flgOpc: '',
	ativo: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'nome': '',
		'flgOpc': '',
		'ativo': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * Guiatp Backbone Collection
 */
model.GuiatpCollection = model.AbstractCollection.extend({
	url: 'api/guiatps',
	model: model.GuiatpModel
});

/**
 * Midia Backbone Model
 */
model.MidiaModel = Backbone.Model.extend({
	urlRoot: 'api/midia',
	idAttribute: 'id',
	id: '',
	idGrp: '',
	titulo: '',
	descricao: '',
	link: '',
	tpMedia: '',
	extensao: '',
	ativo: '',
	visitas: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'idGrp': '',
		'titulo': '',
		'descricao': '',
		'link': '',
		'tpMedia': '',
		'extensao': '',
		'ativo': '',
		'visitas': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * Midia Backbone Collection
 */
model.MidiaCollection = model.AbstractCollection.extend({
	url: 'api/midias',
	model: model.MidiaModel
});

/**
 * MidiaGrp Backbone Model
 */
model.MidiaGrpModel = Backbone.Model.extend({
	urlRoot: 'api/midiagrp',
	idAttribute: 'id',
	id: '',
	nome: '',
	ativo: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'nome': '',
		'ativo': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * MidiaGrp Backbone Collection
 */
model.MidiaGrpCollection = model.AbstractCollection.extend({
	url: 'api/midiagrps',
	model: model.MidiaGrpModel
});

/**
 * Noticias Backbone Model
 */
model.NoticiasModel = Backbone.Model.extend({
	urlRoot: 'api/noticias',
	idAttribute: 'id',
	id: '',
	idSecao: '',
	titulo: '',
	texto: '',
	exporta: '',
	visitas: '',
	destaque: '',
	ativo: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'idSecao': '',
		'titulo': '',
		'texto': '',
		'exporta': '',
		'visitas': '',
		'destaque': '',
		'ativo': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * Noticias Backbone Collection
 */
model.NoticiasCollection = model.AbstractCollection.extend({
	url: 'api/noticiases',
	model: model.NoticiasModel
});

/**
 * NoticiasSec Backbone Model
 */
model.NoticiasSecModel = Backbone.Model.extend({
	urlRoot: 'api/noticiassec',
	idAttribute: 'id',
	id: '',
	nome: '',
	mostrar: '',
	ativo: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'nome': '',
		'mostrar': '',
		'ativo': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * NoticiasSec Backbone Collection
 */
model.NoticiasSecCollection = model.AbstractCollection.extend({
	url: 'api/noticiassecs',
	model: model.NoticiasSecModel
});

/**
 * Noticiasrp Backbone Model
 */
model.NoticiasrpModel = Backbone.Model.extend({
	urlRoot: 'api/noticiasrp',
	idAttribute: 'id',
	id: '',
	idNot: '',
	dataInc: '',
	nome: '',
	email: '',
	comentario: '',
	ativo: '',
	defaults: {
		'id': null,
		'idNot': '',
		'dataInc': new Date(),
		'nome': '',
		'email': '',
		'comentario': '',
		'ativo': ''
	}
});

/**
 * Noticiasrp Backbone Collection
 */
model.NoticiasrpCollection = model.AbstractCollection.extend({
	url: 'api/noticiasrps',
	model: model.NoticiasrpModel
});

/**
 * Os Backbone Model
 */
model.OsModel = Backbone.Model.extend({
	urlRoot: 'api/os',
	idAttribute: 'id',
	id: '',
	nro: '',
	data: '',
	descricao: '',
	status: '',
	defaults: {
		'id': null,
		'nro': '',
		'data': new Date(),
		'descricao': '',
		'status': ''
	}
});

/**
 * Os Backbone Collection
 */
model.OsCollection = model.AbstractCollection.extend({
	url: 'api/oses',
	model: model.OsModel
});

/**
 * Representantes Backbone Model
 */
model.RepresentantesModel = Backbone.Model.extend({
	urlRoot: 'api/representantes',
	idAttribute: 'id',
	id: '',
	nome: '',
	ativo: '',
	telefone: '',
	celular: '',
	email: '',
	usuAlt: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'nome': '',
		'ativo': '',
		'telefone': '',
		'celular': '',
		'email': '',
		'usuAlt': '',
		'dataAlt': ''
	}
});

/**
 * Representantes Backbone Collection
 */
model.RepresentantesCollection = model.AbstractCollection.extend({
	url: 'api/representanteses',
	model: model.RepresentantesModel
});

/**
 * RepresentantesCidades Backbone Model
 */
model.RepresentantesCidadesModel = Backbone.Model.extend({
	urlRoot: 'api/representantescidades',
	idAttribute: 'id',
	id: '',
	idRepresentante: '',
	idCidade: '',
	defaults: {
		'id': null,
		'idRepresentante': '',
		'idCidade': ''
	}
});

/**
 * RepresentantesCidades Backbone Collection
 */
model.RepresentantesCidadesCollection = model.AbstractCollection.extend({
	url: 'api/representantescidadeses',
	model: model.RepresentantesCidadesModel
});

/**
 * Usuarios Backbone Model
 */
model.UsuariosModel = Backbone.Model.extend({
	urlRoot: 'api/usuarios',
	idAttribute: 'id',
	id: '',
	email: '',
	usuario: '',
	senha: '',
	grupo: '',
	flgMaster: '',
	ativo: '',
	acessos: '',
	usuAlt: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'email': '',
		'usuario': '',
		'senha': '',
		'grupo': '',
		'flgMaster': '',
		'ativo': '',
		'acessos': '',
		'usuAlt': '',
		'dataAlt': new Date()
	}
});

/**
 * Usuarios Backbone Collection
 */
model.UsuariosCollection = model.AbstractCollection.extend({
	url: 'api/usuarioses',
	model: model.UsuariosModel
});

/**
 * Videos Backbone Model
 */
model.VideosModel = Backbone.Model.extend({
	urlRoot: 'api/videos',
	idAttribute: 'id',
	id: '',
	titulo: '',
	descricao: '',
	extensao: '',
	link: '',
	visitas: '',
	ativo: '',
	usuInc: '',
	usuAlt: '',
	dataInc: '',
	dataAlt: '',
	defaults: {
		'id': null,
		'titulo': '',
		'descricao': '',
		'extensao': '',
		'link': '',
		'visitas': '',
		'ativo': '',
		'usuInc': '',
		'usuAlt': '',
		'dataInc': new Date(),
		'dataAlt': new Date()
	}
});

/**
 * Videos Backbone Collection
 */
model.VideosCollection = model.AbstractCollection.extend({
	url: 'api/videoses',
	model: model.VideosModel
});

